import 'package:cruiser/src/models/cruiser_transition.dart';
import 'package:flutter/material.dart';
import 'package:flutter/widgets.dart';
import 'dart:math' as math;

class CubeTransition extends CruiserTransition
{
    final Curve curve;
    final double perspectiveScale;
    final double angle;
    final AlignmentGeometry enterPageAlignment;
    final AlignmentGeometry exitPageAlignment;
    final Color backgroundColor;

    CubeTransition({
        this.curve = Curves.easeOutSine,
        this.perspectiveScale = 0.001,
        double angle = 90,
        this.enterPageAlignment = Alignment.centerLeft,
        this.exitPageAlignment = Alignment.centerRight,
        this.backgroundColor = Colors.black,
    }) : this.angle = math.pi / 180 * angle;

    @override
    Widget build(BuildContext context, Animation<double> animation, Widget exitPage, Widget enterPage) 
    {
        animation = CurvedAnimation(parent: animation, curve: curve);

        double width = MediaQuery.of(context).size.width;

        return Stack(
            children: [
                Container(color: backgroundColor),
                    Transform(
                        alignment: exitPageAlignment,
                        transform: Matrix4.identity()
                            ..setEntry(3, 2, perspectiveScale)
                            ..rotateY(angle * (animation.value))
                            ..leftTranslate(-width * (animation.value)),
                        child: exitPage,
                    ),
                    Transform(
                        alignment: enterPageAlignment,
                        transform: Matrix4.identity()
                            ..setEntry(3, 2, perspectiveScale)
                            ..rotateY(-angle * (1 - animation.value))
                            ..leftTranslate(width * (1 - animation.value)),
                        child: enterPage,
                ),
            ],
        );
    }
}