
import 'package:cruiser/src/models/cruiser_transition.dart';
import 'package:flutter/material.dart';
import 'package:flutter/widgets.dart';
import 'dart:math' as math;

class RotateUpTransition extends CruiserTransition
{
    final Curve curve;
    final Color backgroundColor;
    final double angle;

    RotateUpTransition({
        this.curve = Curves.easeOutSine,
        this.backgroundColor = Colors.black,
        double angle = 45,
    }) : this.angle = math.pi / 180 * angle;

    @override
    Widget build(BuildContext context, Animation<double> animation, Widget exitPage, Widget enterPage) 
    {
        animation = CurvedAnimation(parent: animation, curve: curve);
        final double width = MediaQuery.of(context).size.width;
        return Stack(
        children: [
            Container(color: backgroundColor),
            Transform(
            alignment: Alignment.topRight,
            transform: Matrix4.identity()
                ..rotateZ(-angle * animation.value)
                ..leftTranslate(width * animation.value),
            child: exitPage,
            ),
            Transform(
            alignment: Alignment.topLeft,
            transform: Matrix4.identity()
                ..rotateZ(angle * (1.0 - animation.value))
                ..leftTranslate(-width * (1.0 - animation.value)),
            child: enterPage,
            ),
        ],
        );
    }
}
